
var grammar = 
'start = ws module:(object / lambda) ws { return module; };' +
'ws = [ \\t\\n\\r]*;' +
'hws = [ \\t]*;' +
'expr = funcall / addsub;' +
'addsub = left:muldiv hws op:("+"/"-") hws right:addsub ws { return {type: "op", left: left, op: op, right: right }; } / muldiv;'+
'muldiv = left:primlitsym hws op:("*"/"/") hws right:muldiv { return {type: "op", left: left, op: op, right: right }; } / primlitsym;'+
'primlitsym = ws val:(float / int / string / symbol / object / lambda / "(" expr:expr hws ")" { return expr; }) { return val; };' +
'symbol = chars:[a-zA-Z_!?@]+ trailing:(":"? [a-zA-Z_!?@0-9~])* ! ":" { return {type: "symbol", name: chars.join("") + trailing.join("")}; };' +
'float = digits:[0-9]+ "." decimals:[0-9]+ { return {type: "floatlit", value: parseFloat(digits.join("") + "." + decimals.join("")) }; };' +
'int = digits:[0-9]+ { return {type: "intlit", value: parseInt(digits.join(""), 10)}; };' +
'string = "\\"" text:[^\\"]* "\\"" { return {type: "strlit", value: text.join("")}; };' +
'object = "#{" messages:assignment* "}" { return {type: "object", messages: messages}; };' +
'assignment = ws sym:symbol hws "<-" expr:expr { return {type: "assignment", symbol: sym, expression: expr}; }' +
'lambda = args:((& ":") argname+  )? "{" exprs:(assignment / expr)* ws "}" { return {type: "lambda", args: args[1], expressions: exprs}; };' +
'argname = init:":"? chars:[a-zA-Z_!?@]+ trailing:[a-zA-Z_!?@0-9%]* hws { return {type: "symbol", name: init + chars.join("") + trailing.join("")}; };' +
'funcall = ws parts: funcallpart+ { var fun = ""; var args = []; for (var i in parts) { fun += parts[i].name; args = args.concat(parts[i].args); } return {type: "funcall", name: fun, args: args}; };' +
'funcallpart = fun:funpart args:addsub* hws { return { name: fun, args: args}; };' +
'funpart = chars:[a-zA-Z_!?@]+ middle:[a-zA-Z_!?@0-9#]* ":" & [ \\t\\n\\r] { return chars.join("") + middle.join("") + ":"; };';
var parser = PEG.buildParser(grammar);

//var parser = PEG.buildParser('start = expr; expr = int; int = digits:[0-9]+ { return parseInt(digits.join(""), 10); }');

onReady(function() {
	q('input[type=button]').onclick = function() {
		var text = q('textarea').value;
		try {
			var parsed = parser.parse(text);
			q('div').innerHTML = text + "<br><br>" + JSON.stringify(parsed);
			console.log(parsed);
		} catch(e) {
			q('div').innerHTML = e.message + '<br>Line: ' + e.line + '<br>Col: ' + e.column;
		}
	}
});

